@@ -1,8 +1,7 @@
 #include "header.h"
 
-void schlatherindfull(int *covmod, double *data, double *dist, int *nSite,
-		      int *nObs, int *dim, int *weighted, double *weights,
-		      double *locs, double *scales, double *shapes, 
+void schlatherindfull(int *covmod, double *data, double *dist, int *nSite, int *nObs, int *dim,
+		      int *weighted, double *weights, double *locs, double *scales, double *shapes, 
 		      double *alpha, double *sill, double *range, double *smooth,
 		      double *smooth2, int *fitmarge,double *dns){
   //This is the independent Schlater's model. It's a wrapper to several
@@ -32,7 +31,12 @@ void schlatherindfull(int *covmod, double *data, double *dist, int *nSite,
   if ((*alpha < 0) || (*alpha > 1)){
     *dns = MINF;
     return;
-  }    
+  }
+
+  if (*sill > 1){
+    *dns = *sill * *sill * MINF;
+    return;
+  }
    
   //Stage 0: Compute the covariance at each location
   switch (*covmod){
@@ -72,8 +76,7 @@ void schlatherindfull(int *covmod, double *data, double *dist, int *nSite,
   }
   
   else {
-    for (i=0;i<(*nSite * *nObs);i++)
-      jac[i] = 0.0;
+    memset(jac, 0, *nSite * *nObs * sizeof(double));
 
     if (*weighted)
       *dns = wlplikschlatherind(data, *alpha, rho, jac, *nObs, *nSite, weights);
@@ -89,20 +92,28 @@ void schlatherindfull(int *covmod, double *data, double *dist, int *nSite,
 
 }
 
-void schlatherinddsgnmat(int *covmod, double *data, double *dist, int *nSite, int *nObs,
-			 int *dim, int *weighted, double *weights, double *locdsgnmat,
-			 double *locpenmat, int *nloccoeff, int *npparloc,
-			 double *locpenalty, double *scaledsgnmat, double *scalepenmat,
-			 int *nscalecoeff, int *npparscale, double *scalepenalty, double *shapedsgnmat,
-			 double *shapepenmat, int *nshapecoeff, int *npparshape, double *shapepenalty,
-			 double *loccoeff, double *scalecoeff, double *shapecoeff, double *alpha,
-			 double *sill, double *range, double *smooth, double *smooth2, double *dns){
+void schlatherinddsgnmat(int *covmod, double *data, double *dist, int *nSite, int *nObs, int *dim,
+			 int *weighted, double *weights, double *locdsgnmat, double *locpenmat,
+			 int *nloccoeff, int *npparloc, double *locpenalty, double *scaledsgnmat,
+			 double *scalepenmat, int *nscalecoeff, int *npparscale,
+			 double *scalepenalty, double *shapedsgnmat, double *shapepenmat,
+			 int *nshapecoeff, int *npparshape, double *shapepenalty, int *usetempcov,
+			 double *tempdsgnmatloc, double *temppenmatloc, int *ntempcoeffloc,
+			 int *nppartempcoeffloc, double *temppenaltyloc, double *tempdsgnmatscale,
+			 double *temppenmatscale, int *ntempcoeffscale, int *nppartempcoeffscale,
+			 double *temppenaltyscale, double *tempdsgnmatshape, double *temppenmatshape,
+			 int *ntempcoeffshape, int *nppartempcoeffshape, double *temppenaltyshape,
+			 double *loccoeff, double *scalecoeff, double *shapecoeff,
+			 double *tempcoeffloc, double *tempcoeffscale, double *tempcoeffshape,
+			 double *alpha, double *sill, double *range, double *smooth,
+			 double *smooth2, double *dns){
   //This is the independent Schlater's model.
   //The GEV parameters are defined using a polynomial response surface
   //or p-splines.
   
   const int nPairs = *nSite * (*nSite - 1) / 2;
-  double *jac, *rho, *locs, *scales, *shapes, *frech;
+  int flag = usetempcov[0] + usetempcov[1] + usetempcov[2];
+  double *jac, *rho, *locs, *scales, *shapes, *frech, *trendlocs, *trendscales, *trendshapes;
     
   jac = (double *)R_alloc(*nObs * *nSite, sizeof(double));
   rho = (double *)R_alloc(nPairs, sizeof(double));
@@ -116,6 +127,11 @@ void schlatherinddsgnmat(int *covmod, double *data, double *dist, int *nSite, in
     return;
   }
 
+  if (*sill > 1){
+    *dns = *sill * *sill * MINF;
+    return;
+  }
+
   //Stage 1: Compute the covariance at each location
   switch (*covmod){
   case 1:
@@ -139,17 +155,37 @@ void schlatherinddsgnmat(int *covmod, double *data, double *dist, int *nSite, in
     return;
   
   //Stage 2: Compute the GEV parameters using the design matrix
-  *dns = dsgnmat2Param(locdsgnmat, scaledsgnmat, shapedsgnmat,
-		       loccoeff, scalecoeff, shapecoeff, *nSite,
-		       *nloccoeff, *nscalecoeff, *nshapecoeff,
-		       locs, scales, shapes);
+  *dns = dsgnmat2Param(locdsgnmat, scaledsgnmat, shapedsgnmat, loccoeff, scalecoeff, shapecoeff,
+		       *nSite, *nloccoeff, *nscalecoeff, *nshapecoeff, locs, scales, shapes);
 
-  if (*dns != 0.0)
+  if (flag){
+    int i, j;
+    trendlocs = (double *)R_alloc(*nObs, sizeof(double));
+    trendscales = (double *)R_alloc(*nObs, sizeof(double));
+    trendshapes = (double *)R_alloc(*nObs, sizeof(double));
+
+    dsgnmat2temptrend(tempdsgnmatloc, tempdsgnmatscale, tempdsgnmatshape, tempcoeffloc,
+		      tempcoeffscale, tempcoeffshape, *nSite, *nObs, usetempcov, *ntempcoeffloc,
+		      *ntempcoeffscale, *ntempcoeffshape, trendlocs, trendscales, trendshapes);
+
+    for (i=*nSite;i--;)
+      for (j=*nObs;j--;)
+	if (((scales[i] + trendscales[j]) <= 0) || ((shapes[i] + trendshapes[j]) <= -1)){
+	  *dns = MINF;
+	  return;
+	}
+  }
+
+  else if (*dns != 0.0)
     return;
 
   //Stage 3: Transformation to unit Frechet
-  *dns = gev2frech(data, *nObs, *nSite, locs, scales, shapes,
-		   jac, frech);
+  if (flag)
+    *dns = gev2frechTrend(data, *nObs, *nSite, locs, scales, shapes, trendlocs, trendscales,
+			  trendshapes, jac, frech);
+
+  else
+    *dns = gev2frech(data, *nObs, *nSite, locs, scales, shapes, jac, frech);
 
   if (*dns != 0.0)
     return;
@@ -163,19 +199,29 @@ void schlatherinddsgnmat(int *covmod, double *data, double *dist, int *nSite, in
   //Stage 5: Removing the penalizing terms (if any)
   // 1- For the location parameter
   if (*locpenalty > 0)
-    *dns -= penalization(locpenmat, loccoeff, *locpenalty,
-			 *nloccoeff, *npparloc);
+    *dns -= penalization(locpenmat, loccoeff, *locpenalty, *nloccoeff, *npparloc);
   
   // 2- For the scale parameter
   if (*scalepenalty > 0)    
-    *dns -= penalization(scalepenmat, scalecoeff, *scalepenalty,
-			 *nscalecoeff, *npparscale);
+    *dns -= penalization(scalepenmat, scalecoeff, *scalepenalty, *nscalecoeff, *npparscale);
   
   // 3- For the shape parameter
   if (*shapepenalty > 0)
-    *dns -= penalization(shapepenmat, shapecoeff, *shapepenalty,
-			 *nshapecoeff, *npparshape);
+    *dns -= penalization(shapepenmat, shapecoeff, *shapepenalty, *nshapecoeff, *npparshape);
   
+  // 4- Doing the same thing for the temporal component
+  if (*temppenaltyloc > 0)
+    *dns -= penalization(temppenmatloc, tempcoeffloc, *temppenaltyloc, *ntempcoeffloc,
+			 *nppartempcoeffloc);
+
+  if (*temppenaltyscale > 0)
+    *dns -= penalization(temppenmatscale, tempcoeffscale, *temppenaltyscale, *ntempcoeffscale,
+			 *nppartempcoeffscale);
+
+  if (*temppenaltyshape > 0)
+    *dns -= penalization(temppenmatshape, tempcoeffshape, *temppenaltyshape, *ntempcoeffshape,
+			 *nppartempcoeffshape);
+
   if (!R_FINITE(*dns))
     *dns = MINF;
 
