@@ -1,29 +1,25 @@
 #include "header.h"
+/* WARNING: There's a bias in the simulation of fbm!!! */
 
 void tbm(int *nobs, int *nsite, int *dim, int *covmod, int *grid, 
 	 double *coord, double *nugget, double *sill, double *range,
 	 double *smooth, int *nlines, double *ans){
   
-  const double normConst = sqrt(2 * *sill / *nlines),
-    sdnugget = sqrt(*nugget);
-  int i, j, k, l, m, ngrid = *nsite;
-  double *lines, freq, eucProd, u1, u2, G, phase,
-    irange = 1 / *range, u, v, w, inorm, angle;
-
+  double normConst = sqrt(*sill / *nlines), sdnugget = sqrt(*nugget);
+  int i, j, k, l, m, neffSite = *nsite;
+  double freq, eucProd, u1, u2, G, phase, irange = 1 / *range, u, v, w,
+    inorm, angle, r, theta;
+ 
   //rescale the coordinates
   for (i=(*nsite * *dim);i--;)
     coord[i] = coord[i] * irange;
 
-
   if (*grid)
-    for (i=(*nobs * R_pow_di(*nsite, *dim));i--;)
-      ans[i] = 0;
+    neffSite = R_pow_di(neffSite, *dim);
 
-  else
-    for (i=(*nobs * *nsite);i--;)
-      ans[i] = 0;
-
-  lines = (double *)R_alloc(3 * *nlines, sizeof(double));
+  memset(ans, 0, *nobs * neffSite * sizeof(double));
+    
+  double *lines = (double *)R_alloc(3 * *nlines, sizeof(double));
   
   if ((*covmod == 3) && (*smooth == 2))
     //This is the gaussian case
@@ -35,8 +31,7 @@ void tbm(int *nobs, int *nsite, int *dim, int *covmod, int *grid,
   GetRNGstate();
   if (*grid){
     //coord defines a grid
-    ngrid = R_pow_di(*nsite, *dim);
-
+    
     for (i=*nobs;i--;){
       //Random rotation of the lines
       u = unif_rand() - 0.5;
@@ -66,7 +61,7 @@ void tbm(int *nobs, int *nsite, int *dim, int *covmod, int *grid,
 	      cl = coord[k] * lines[*nlines + j];
 	      for (l=*nsite;l--;){
 		eucProd =  cl + coord[*nsite + l] * lines[2 * *nlines + j];
-		ans[i * ngrid + k  + l * *nsite] += cos(freq * eucProd + phase);
+		ans[i * neffSite + k  + l * *nsite] += cos(freq * eucProd + phase);
 	      }
 	    }
 	  }
@@ -81,7 +76,7 @@ void tbm(int *nobs, int *nsite, int *dim, int *covmod, int *grid,
 	      cl = coord[k] * lines[*nlines + j];
 	      for (l=*nsite;l--;){
 	      eucProd = cl + coord[*nsite + l] * lines[2 * *nlines + j];
-	      ans[i * ngrid + k  + l * *nsite] += cos(freq * eucProd + phase);
+	      ans[i * neffSite + k  + l * *nsite] += cos(freq * eucProd + phase);
 	      }
 	    }
 	  }
@@ -91,18 +86,18 @@ void tbm(int *nobs, int *nsite, int *dim, int *covmod, int *grid,
 	  for (j=*nlines;j--;){
 	    u1 = rexp(1);
 	    u2 = runif(-M_PI_2, M_PI_2);
-	    G = fabs(sin(0.5 * *smooth * (u2 - M_PI_2)) * R_pow(cos(u2), -1 / *smooth) *
+	    G = fabs(sin(0.5 * *smooth * (u2 - M_PI_2)) * R_pow(cos(u2), -2 / *smooth) *
 		     R_pow(cos(u2 - 0.5 * *smooth * (u2 - M_PI_2)) / u1, 
 			   (2 - *smooth) / *smooth));
 	
-	    freq = sqrt(rchisq(3) * M_SQRT_3 / G);
+	    freq = sqrt(2 * rchisq(3) * G);
 	    phase = M_2PI * unif_rand();
 	
 	    for (k=*nsite;k--;){
 	      cl = coord[k] * lines[*nlines + j];
 	      for (l=*nsite;l--;){
 		eucProd = cl + coord[*nsite + l] * lines[2 * *nlines + j];
-		ans[i * ngrid + k  + l * *nsite] += cos(freq * eucProd + phase);
+		ans[i * neffSite + k  + l * *nsite] += cos(freq * eucProd + phase);
 	      }
 	    }
 	  }
@@ -110,14 +105,14 @@ void tbm(int *nobs, int *nsite, int *dim, int *covmod, int *grid,
 	case 4:
 	  //Bessel
 	  for (j=*nlines;j--;){
-	    freq = sqrt(beta(1.5, *smooth - 0.5) * *range);
+	    freq = sqrt(beta(1.5, *smooth - 0.5));
 	    phase = M_2PI * unif_rand();
 	
 	    for (k=*nsite;k--;){
 	      cl = coord[k] * lines[*nlines + j];
 	      for (l=*nsite;l--;){
 		eucProd = cl + coord[*nsite + l] * lines[2 * *nlines + j];
-		ans[i * ngrid + k  + l * *nsite] += cos(freq * eucProd + phase);
+		ans[i * neffSite + k  + l * *nsite] += cos(freq * eucProd + phase);
 	      }
 	    }
 	  }
@@ -132,11 +127,39 @@ void tbm(int *nobs, int *nsite, int *dim, int *covmod, int *grid,
 	      cl = coord[k] * lines[*nlines + j];
 	      for (l=*nsite;l--;){
 		eucProd = cl + coord[*nsite + l] * lines[2 * *nlines + j];
-		ans[i * ngrid + k  + l * *nsite] += cos(freq * eucProd + phase);
+		ans[i * neffSite + k  + l * *nsite] += cos(freq * eucProd + phase);
 	      }
 	    }
 	  }
 	  break;
+	case 6:
+	  //Fractional brownian motion
+	  for (j=*nlines;j--;){
+	    r = rgamma(1 - 0.5 * *smooth, 1) / rgamma(0.5 * *smooth, 1);
+	    theta = sqrt((1 + r) / R_pow(r, 0.5 * *smooth + 1));
+	    freq = M_2PI * r;
+	    phase = M_2PI * unif_rand();
+
+	    for (k=*nsite;k--;){
+	      cl = coord[k] * lines[*nlines + j];
+	      for (l=*nsite;l--;){
+		eucProd = cl + coord[*nsite + l] * lines[2 * *nlines + j];
+		ans[i * neffSite + k + l * *nsite] += theta * cos(freq * eucProd + phase);
+	      }
+	    }
+
+	    if (r < 1e-3){
+	      double cosPhase = cos(phase);
+	      for (k=*nsite;k--;){
+		cl = coord[k] * lines[*nlines + j];
+		for (l=*nsite;l--;){
+		  eucProd = cl + coord[*nsite + l] * lines[2 * *nlines + j];
+		  ans[i * neffSite + k + l * *nsite] -= theta * cosPhase;
+		}
+	      }
+	      }
+	  }
+	  break;
 	}
       }
 
@@ -155,7 +178,7 @@ void tbm(int *nobs, int *nsite, int *dim, int *covmod, int *grid,
 		cl2 = coord[*nsite + l] * lines[*nlines + j];
 		for (m=*nsite;m--;){
 		  eucProd = cl1 + cl2 + coord[2 * *nsite + m] * lines[2 * *nlines + j];
-		  ans[i * ngrid + k + *nsite * ( l + m * *nsite)] += cos(freq * eucProd + phase);
+		  ans[i * neffSite + k + *nsite * ( l + m * *nsite)] += cos(freq * eucProd + phase);
 		}
 	      }
 	    }
@@ -173,7 +196,7 @@ void tbm(int *nobs, int *nsite, int *dim, int *covmod, int *grid,
 		cl2 = coord[*nsite + l] * lines[*nlines + j];
 		for (m=*nsite;m--;){
 		  eucProd = cl1 + cl2 + coord[2 * *nsite + m] * lines[2 * *nlines + j];
-		  ans[i * ngrid + k + *nsite * ( l + m * *nsite)] += cos(freq * eucProd + phase);
+		  ans[i * neffSite + k + *nsite * ( l + m * *nsite)] += cos(freq * eucProd + phase);
 		}
 	      }
 	    }
@@ -184,11 +207,11 @@ void tbm(int *nobs, int *nsite, int *dim, int *covmod, int *grid,
 	  for (j=*nlines;j--;){
 	    u1 = rexp(1);
 	    u2 = runif(-M_PI_2, M_PI_2);
-	    G = fabs(sin(0.5 * *smooth * (u2 - M_PI_2)) * R_pow(cos(u2), -1 / *smooth) *
+	    G = fabs(sin(0.5 * *smooth * (u2 - M_PI_2)) * R_pow(cos(u2), -2 / *smooth) *
 		     R_pow(cos(u2 - 0.5 * *smooth * (u2 - M_PI_2)) / u1, 
 			   (2 - *smooth) / *smooth));
 	
-	    freq = sqrt(rchisq(3) * M_SQRT_3 / G);
+	    freq = sqrt(2 * rchisq(3) * G);
 	    phase = M_2PI * unif_rand();
 	
 	    for (k=*nsite;k--;){
@@ -197,7 +220,7 @@ void tbm(int *nobs, int *nsite, int *dim, int *covmod, int *grid,
 		cl2 = coord[*nsite + l] * lines[*nlines + j];
 		for (m=*nsite;m--;){
 		  eucProd = cl1 + cl2 + coord[2 * *nsite + m] * lines[2 * *nlines + j];
-		  ans[i * ngrid + k + *nsite * ( l + m * *nsite)] += cos(freq * eucProd + phase);
+		  ans[i * neffSite + k + *nsite * ( l + m * *nsite)] += cos(freq * eucProd + phase);
 		}
 	      }
 	    }
@@ -206,7 +229,7 @@ void tbm(int *nobs, int *nsite, int *dim, int *covmod, int *grid,
 	case 4:
 	  //Bessel
 	  for (j=*nlines;j--;){
-	    freq = sqrt(beta(1.5, *smooth - 0.5) * *range);
+	    freq = sqrt(beta(1.5, *smooth - 0.5));
 	    phase = M_2PI * unif_rand();
 	
 	    for (k=*nsite;k--;){
@@ -215,7 +238,7 @@ void tbm(int *nobs, int *nsite, int *dim, int *covmod, int *grid,
 		cl2 = coord[*nsite + l] * lines[*nlines + j];
 		for (m=*nsite;m--;){
 		  eucProd = cl1 + cl2 + coord[2 * *nsite + m] * lines[2 * *nlines + j];
-		  ans[i * ngrid + k + *nsite * ( l + m * *nsite)] += cos(freq * eucProd + phase);
+		  ans[i * neffSite + k + *nsite * ( l + m * *nsite)] += cos(freq * eucProd + phase);
 		}
 	      }
 	    }
@@ -233,11 +256,46 @@ void tbm(int *nobs, int *nsite, int *dim, int *covmod, int *grid,
 		cl2 = coord[*nsite + l] * lines[*nlines + j];
 		for (m=*nsite;m--;){
 		  eucProd = cl1 + cl2 + coord[2 * *nsite + m] * lines[2 * *nlines + j];
-		  ans[i * ngrid + k + *nsite * ( l + m * *nsite)] += cos(freq * eucProd + phase);
+		  ans[i * neffSite + k + *nsite * ( l + m * *nsite)] += cos(freq * eucProd + phase);
 		}
 	      }
 	    }
-	  }    
+	  }
+	  break;
+	  case 6:
+	  //Fractional brownian motion
+	  for (j=*nlines;j--;){
+	    r = rgamma(1 - 0.5 * *smooth, 1) / rgamma(0.5 * *smooth, 1);
+	    theta = sqrt((1 + r) / R_pow(r, 0.5 * *smooth + 1));
+	    freq = M_2PI * r;
+	    phase = M_2PI * unif_rand();
+
+	    for (k=*nsite;k--;){
+	      cl1 = coord[k] * lines[*nlines + j];
+	      for (l=*nsite;l--;){
+		cl2 = coord[*nsite + l] * lines[*nlines + j];
+		for (m=*nsite;m--;){
+		  eucProd = cl1 + cl2 + coord[2 * *nsite + m] * lines[2 * *nlines + j];
+		  ans[i * neffSite + k + *nsite * (l + m * *nsite)] += theta * cos(freq * eucProd + phase);
+		}
+	      }
+	    }
+
+	    if (r < 1e-3){
+	      double cosPhase = cos(phase);
+	      for (k=*nsite;k--;){
+		cl1 = coord[k] * lines[*nlines + j];
+		for (l=*nsite;l--;){
+		  cl2 = coord[*nsite + l] * lines[*nlines + j];
+		  for (m=*nsite;m--;){
+		    eucProd = cl1 + cl2 + coord[2 * *nsite + l] * lines[2 * *nlines + j];
+		    ans[i * neffSite + k + *nsite * (l + m * *nsite)] -= theta * cosPhase;
+		  }
+		}
+	      }
+	    }
+	  }
+	  break;
 	}
       }
     }  
@@ -292,11 +350,11 @@ void tbm(int *nobs, int *nsite, int *dim, int *covmod, int *grid,
 	  for (j=*nlines;j--;){
 	    u1 = rexp(1);
 	    u2 = runif(-M_PI_2, M_PI_2);
-	    G = fabs(sin(0.5 * *smooth * (u2 - M_PI_2)) * R_pow(cos(u2), -1 / *smooth) *
+	    G = fabs(sin(0.5 * *smooth * (u2 - M_PI_2)) * R_pow(cos(u2), -2 / *smooth) *
 		     R_pow(cos(u2 - 0.5 * *smooth * (u2 - M_PI_2)) / u1, 
 			   (2 - *smooth) / *smooth));
 	
-	    freq = sqrt(rchisq(3) * M_SQRT_3 / G);
+	    freq = sqrt(2 * rchisq(3) * G);
 	    phase = M_2PI * unif_rand();
 	
 	    for (k=*nsite;k--;){
@@ -308,7 +366,7 @@ void tbm(int *nobs, int *nsite, int *dim, int *covmod, int *grid,
 	case 4:
 	  //Bessel
 	  for (j=*nlines;j--;){
-	    freq = sqrt(beta(1.5, *smooth - 0.5) * *range);
+	    freq = sqrt(beta(1.5, *smooth - 0.5));
 	    phase = M_2PI * unif_rand();
 	
 	    for (k=*nsite;k--;){
@@ -329,6 +387,28 @@ void tbm(int *nobs, int *nsite, int *dim, int *covmod, int *grid,
 	    }
 	  }
 	  break;
+	  case 6:
+	  //Fractional brownian motion
+	  for (j=*nlines;j--;){
+	    r = rgamma(1 - 0.5 * *smooth, 1) / rgamma(0.5 * *smooth, 1);
+	    theta = sqrt((1 + r) / R_pow(r, 0.5 * *smooth + 1));
+	    freq = M_2PI * r;
+	    phase = M_2PI * unif_rand();
+
+	    for (k=*nsite;k--;){
+	      eucProd = coord[k] * lines[*nlines + j] + coord[*nsite + k] * lines[2 * *nlines + j];
+	      ans[i + k * *nobs] += theta * cos(freq * eucProd + phase);
+	      }
+
+	    if (r < 1e-3){
+	      double cosPhase = cos(phase);
+	      for (k=*nsite;k--;){
+		eucProd = coord[k] * lines[*nlines + j] + coord[*nsite + k] * lines[2 * *nlines + j];
+		ans[i + k * *nobs] -= theta * cosPhase;
+	      }
+	    }
+	  }
+	  break;
 	}
       }
 
@@ -365,11 +445,11 @@ void tbm(int *nobs, int *nsite, int *dim, int *covmod, int *grid,
 	  for (j=*nlines;j--;){
 	    u1 = rexp(1);
 	    u2 = runif(-M_PI_2, M_PI_2);
-	    G = fabs(sin(0.5 * *smooth * (u2 - M_PI_2)) * R_pow(cos(u2), -1 / *smooth) *
+	    G = fabs(sin(0.5 * *smooth * (u2 - M_PI_2)) * R_pow(cos(u2), -2 / *smooth) *
 		     R_pow(cos(u2 - 0.5 * *smooth * (u2 - M_PI_2)) / u1, 
 			   (2 - *smooth) / *smooth));
 	
-	    freq = sqrt(rchisq(3) * M_SQRT_3 / G);
+	    freq = sqrt(2 * rchisq(3)* G);
 	    phase = M_2PI * unif_rand();
 	
 	    for (k=*nsite;k--;){
@@ -382,7 +462,7 @@ void tbm(int *nobs, int *nsite, int *dim, int *covmod, int *grid,
 	case 4:
 	  //Bessel
 	  for (j=*nlines;j--;){
-	    freq = sqrt(beta(1.5, *smooth - 0.5) * *range);
+	    freq = sqrt(beta(1.5, *smooth - 0.5));
 	    phase = M_2PI * unif_rand();
 	
 	    for (k=*nsite;k--;){
@@ -403,17 +483,50 @@ void tbm(int *nobs, int *nsite, int *dim, int *covmod, int *grid,
 		coord[2 * *nsite + k] * lines[2 * *nlines + j];
 	      ans[i + k * *nobs] += cos(freq * eucProd + phase);
 	    }
-	  }    
+	  }
+	  break;
+	case 6:
+	  //Fractional brownian motion
+	  for (j=*nlines;j--;){
+	    r = rgamma(1 - 0.5 * *smooth, 1) / rgamma(0.5 * *smooth, 1);
+	    theta = sqrt((1 + r) / R_pow(r, 0.5 * *smooth + 1));
+	    freq = M_2PI * r;
+	    phase = M_2PI * unif_rand();
+
+	    for (k=*nsite;k--;){
+	      eucProd = coord[k] * lines[j] + coord[*nsite + k] * lines[*nlines + j] +
+		coord[2 * *nsite + k] * lines[2 * *nlines + j];
+	      ans[i * k * *nobs] += theta * cos(freq * eucProd + phase);
+	      }
+
+	    if (r < 1e-3){
+	      double cosPhase = cos(phase);
+	      for (k=*nsite;k--;){
+		eucProd = coord[k] * lines[j] + coord[*nsite + k] * lines[*nlines + j] +
+		  coord[2 * *nsite + k] * lines[2 * *nlines + j];
+		ans[i + k * *nobs] -= theta * cosPhase;
+	      }
+	    }
+	  }
+	  break;
 	}
       }
     }  
   }
+  
+  if (*covmod != 6)
+    normConst *= M_SQRT2;
 
-  for (i=(ngrid * *nobs);i--;)
+  else
+    normConst *= sqrt(4 * gammafn(0.5 * *smooth + 1) * 
+		      gammafn(0.5 * (*dim + *smooth)) /
+		      (R_pow(M_PI, *smooth) * gammafn(0.5 * *dim)));
+
+  for (i=(neffSite * *nobs);i--;)
     ans[i] *= normConst;
 
   if (*nugget != 0)
-    for (i=(ngrid * *nobs);i--;)
+    for (i=(neffSite * *nobs);i--;)
       ans[i] += sdnugget * norm_rand();
   
   PutRNGstate();
